create or replace function func_trig_bt_access_entry_id() returns trigger as $$
declare
    i_entry_id    integer;        -- 投稿ID
    i_blog_sys_id integer;        -- ブログ投稿別アクセスレコード
    s_access_date char(8);        -- アクセス日時
begin
    -- INSERT
    if (tg_op = 'INSERT') then

        -- ブログのトップページを参照？
        if (new.entry_id = '-1') then

            -- 最新の投稿情報を取得
            select
                entry_id
                into i_entry_id
            from
                bt_entry
            where
                blog_sys_id = new.blog_sys_id
            and
               (public_flg='0' OR (public_flg='2' AND public_date <= now() ) )
            and 
                admit='1'
            order by
                entry_date desc, entry_id desc
            limit 1
                ;

            -- 最新の投稿情報が存在しない
            if (i_entry_id <= 0) then
                new.entry_id := null;
            else 
                new.entry_id := i_entry_id;
            end if;
        -- その他のページ
        else

            -- 最新の投稿情報を取得
            select
                count(*)
                into i_entry_id
            from
                bt_entry
            where
                blog_sys_id = new.blog_sys_id
            and
                entry_id = new.entry_id
            and
               (public_flg='0' OR (public_flg='2' AND public_date <= now() ) )
            and 
                admit='1';

            -- 記事別ページ有
            if (i_entry_id <= '0') then
                new.entry_id := null;
            end if;
        end if;

        -- 投稿IDが存在する？
        if (new.entry_id > '0') then

            -- アクセス日時を設定する
            s_access_date := TO_CHAR(new.access_date, 'YYYYMMDD'); 

            -- 投稿別アクセステーブルのレコードを取得
            select
                blog_sys_id
                into i_blog_sys_id
            from
                bt_blog_entry_access
            where
                blog_sys_id = new.blog_sys_id
                and entry_id = new.entry_id
                and access_date = s_access_date
            for update;

            -- 行が存在する
            if (i_blog_sys_id > 0) then

                -- 投稿別アクセス数を＋１する
                update
                    bt_blog_entry_access
                set
                    access_count = access_count + 1
                where
                    blog_sys_id = new.blog_sys_id
                    and entry_id = new.entry_id
                    and access_date = s_access_date;
            -- レコードなし
            else
                begin
                    -- 投稿別アクセス数データを登録する
                    insert into
                        bt_blog_entry_access
                    (
                        blog_sys_id
                        , entry_id
                        , access_date
                        , access_count
                    ) values (
                        new.blog_sys_id
                        , new.entry_id
                        , s_access_date
                        , 1
                    );

                -- 二重登録でエラーの場合
                exception when unique_violation then
                    -- 何もしない
                end;
            end if;
        end if;
    end if;

    -- 正常終了
    return new;
end;
$$ language plpgsql;