<?php
//------------------------------------------------------------------------------
//--                                                                          --
//--    FILE NAME :  NeoMailer.class.php                                      --
//--                                                                          --
//--        Designed by NeoJapan   2005-11-27   Ver. 1.00                     --
//--        Coded    by NeoJapan   2005-11-27   Ver. 1.00                     --
//--        Revised  by XXXXXXXXX  XXXX-XX-XX   Ver. X.00                     --
//--                                                                          --
//------------------------------------------------------------------------------
//--   (C)COPYRIGHT 2005 BY NeoJapan CORPORATION ALL RIGHTS RESERVED          --
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// インクルード
//------------------------------------------------------------------------------
require_once('phpmailer' . DIRECTORY_SEPARATOR . 'class.phpmailer.php');

//------------------------------------------------------------------------------
// 定数定義
//------------------------------------------------------------------------------
define('JIS_ESC_START', "\x1B".'$B');
define('JIS_ESC_END'  , "\x1B(B");
define('NEOMAILER_MAX_LINE_LENGTH', 300);
//------------------------------------------------------------------------------
// クラス定義
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//--    クラス： NeoMailer                                                    --
//--    Class ： NeoMailer                                                    --
//------------------------------------------------------------------------------
//--  MODIFIED        YYYY-MM-DD REV    - NOTES                               --
//--  NEOJAPAN        2005-11-27 1.00   - CREATE NEW                          --
//------------------------------------------------------------------------------
/**
 *
 *     概要 :
 *     備考 : php4.3.0以降
 */
class NeoMailer extends PHPMailer
{
    /**
     * phpスクリプト内で扱う文字コード
     * @var string
     */
    var $ScriptCharset = "";

    //--------------------------------------------------------------------------
    // インスタンスメソッド
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    //--    関数名： メール送信                                               --
    //--    Method： MailSend()                                               --
    //--------------------------------------------------------------------------
    //--  MODIFIED        YYYY-MM-DD REV    - NOTES                           --
    //--  NEOJAPAN        2005-11-27 1.00   - CREATE NEW                      --
    //--------------------------------------------------------------------------
    /**
     * PHP mail()関数でメール送信をする。
     *
     * @access private
     * @param  string  $header [IN]:メールヘッダー
     * @param  string  $body   [IN]:メール本文
     * @return bool    成功したらtrue、そうでなければfalse
     */
    function MailSend($header, $body)
    {
        $to_list = array();

        // Toの送信先をそれぞれエンコード
        for($i = 0; $i < count($this->to); $i++)
        {
            $to_list[] = $this->AddrFormat($this->to[$i]);
        }
        // Toの送信先を連結
        $to = implode(',', $to_list);

        // 以下、親クラスと同じ処理
        if ($this->Sender != "" && strlen(ini_get("safe_mode"))< 1)
        {
            $old_from = ini_get("sendmail_from");
            ini_set("sendmail_from", $this->Sender);
            $params = sprintf("-oi -f %s", $this->Sender);
            $rt = @mail($to, $this->EncodeHeader($this->Subject), $body,
                        $header, $params);
        }
        else
        {
            $rt = @mail($to, $this->EncodeHeader($this->Subject), $body, $header);
        }

        if (isset($old_from))
            ini_set("sendmail_from", $old_from);

        if(!$rt)
        {
            $this->SetError($this->Lang("instantiate"));
            return false;
        }

        return true;
    }

    //--------------------------------------------------------------------------
    //--    関数名： 本文の作成                                               --
    //--    Method： CreateBody()                                             --
    //--------------------------------------------------------------------------
    //--  MODIFIED        YYYY-MM-DD REV    - NOTES                           --
    //--  NEOJAPAN        2005-11-27 1.00   - CREATE NEW                      --
    //--------------------------------------------------------------------------
    /**
     * 本文を作成する。
     * 本文(body)のみを文字コード変換をかける。
     * @access private
     */
    function CreateBody() {
        $result = "";
        if($this->message_type == "plain"){
            $body = $this->ConvertString($this->rm_insertCR($this->Body));
        }
        else {
            $body = $this->ConvertString($this->Body);
        }

        $this->SetWordWrap();

        switch($this->message_type)
        {
            case "alt":
                $result .= $this->GetBoundary($this->boundary[1], "",
                                              "text/plain", "");
                $result .= $this->EncodeString($this->AltBody, $this->Encoding);
                $result .= $this->LE.$this->LE;
                $result .= $this->GetBoundary($this->boundary[1], "",
                                              "text/html", "");

                $result .= $this->EncodeString($body, $this->Encoding);
                $result .= $this->LE.$this->LE;

                $result .= $this->EndBoundary($this->boundary[1]);
                break;
            case "plain":
                $result .= $this->EncodeString($body, $this->Encoding);
                break;
            case "attachments":
                $result .= $this->GetBoundary($this->boundary[1], "", "", "");
                $result .= $this->EncodeString($body, $this->Encoding);
                $result .= $this->LE;

                $result .= $this->AttachAll();
                break;
            case "alt_attachments":
                $result .= sprintf("--%s%s", $this->boundary[1], $this->LE);
                $result .= sprintf("Content-Type: %s;%s" .
                                   "\tboundary=\"%s\"%s",
                                   "multipart/alternative", $this->LE,
                                   $this->boundary[2], $this->LE.$this->LE);

                // Create text body
                $result .= $this->GetBoundary($this->boundary[2], "",
                                              "text/plain", "") . $this->LE;

                $result .= $this->EncodeString($this->AltBody, $this->Encoding);
                $result .= $this->LE.$this->LE;

                // Create the HTML body
                $result .= $this->GetBoundary($this->boundary[2], "",
                                              "text/html", "") . $this->LE;

                $result .= $this->EncodeString($body, $this->Encoding);
                $result .= $this->LE.$this->LE;

                $result .= $this->EndBoundary($this->boundary[2]);

                $result .= $this->AttachAll();
                break;
        }
        if($this->IsError())
            $result = "";

        return $result;
    }

    //--------------------------------------------------------------------------
    //--    関数名： ヘッダ用エンコード処理                                   --
    //--    Method： EncodeHeader()                                           --
    //--------------------------------------------------------------------------
    //--  MODIFIED        YYYY-MM-DD REV    - NOTES                           --
    //--  NEOJAPAN        2005-07-11 1.00   - CREATE NEW                      --
    //--------------------------------------------------------------------------
    /**
     *
     * Encode a header string to best of Q, B, quoted or none.
     * @access private
     * @return string
     */
    function EncodeHeader ($str, $position = 'text')
    {
        $x = 0;

        $str = $this->ConvertString($str);

        switch (strtolower($position))
        {
            case 'phrase':
                // 語句をエンコードする場合、アルファベット、数字、一部記号以外の文字の数を数える。
                $x = preg_match_all('/[^A-Za-z0-9_.\-]/', $str, $matches);
                break;
            case 'comment':
                $x = preg_match_all('/[()"]/', $str, $matches);
                // Fall-through
            case 'text':
            default:
                // 文章をエンコードする場合、アルファベット、数字、一部記号以外の文字の数を数える。
                $x += preg_match_all('/[\000-\010\013\014\016-\037\177-\377]/', $str, $matches);
                break;
        }

        // 変換文字がないときは変換しない
        if ($x == 0)
        {
            return ($str);
        }

        //[ JISならば先頭のエスケープを追加 ]
        if (strcasecmp($this->CharSet, 'ISO-2022-JP' == 0))
        {
            return $this->rm_jis_b_mime_header($str, $this->LE);
        }

        // Bエンコード
        $maxlen = 75 - 7 - strlen($this->CharSet);
        $maxlen = ((int) ($maxlen / 4)) * 3;
        $encoding = 'B';
        $chunked = trim($this->bm_chunk_split($str, $maxlen, "\n"));
        $encoded = base64_encode($chunked);

        $encoded = preg_replace('/^(.*)$/m', " =?".$this->CharSet."?$encoding?\\1?=", $encoded);
        $encoded = trim(str_replace("\n", $this->LE, $encoded));

        return $encoded;
    }

    //--------------------------------------------------------------------------
    //--    関数名： 文字コード変換                                           --
    //--    Method： ConvertString()                                          --
    //--------------------------------------------------------------------------
    //--  MODIFIED        YYYY-MM-DD REV    - NOTES                           --
    //--  NEOJAPAN        2005-11-27 1.00   - CREATE NEW                      --
    //--------------------------------------------------------------------------
    /**
     * 文字コードを変換する。
     *
     * @access private
     * @param  string  $str        [IN]:変換前文字列
     * @return string  変換後文字列
     */
    function ConvertString($str)
    {
        // 指定された文字コードでエンコード
        if (function_exists('mb_convert_encoding') && $this->ScriptCharset)
        {
            $str = mb_convert_encoding($str, $this->CharSet, $this->ScriptCharset);
        }
        // 内部エンコードでエンコード
        else if (function_exists('mb_convert_encoding'))
        {
            $str = mb_convert_encoding($str, $this->CharSet);
        }

        return $str;
    }

    //--------------------------------------------------------------------------
    //--    関数名： 文字コードを変換する                                     --
    //--    Method： rm_jis_b_mime_header()                                   --
    //--------------------------------------------------------------------------
    //--  MODIFIED        YYYY-MM-DD REV    - NOTES                           --
    //--  NEOJAPAN        2005-11-27 1.00   - CREATE NEW                      --
    //--------------------------------------------------------------------------
    /**
     * 文字コードを変換する。
     *
     * @access private
     * @param  string  $str        [IN]:変換前文字列
     * @return string  変換後文字列
     */
    function rm_jis_b_mime_header($str, $ending = "\r\n")
    {
        $cnt        = 0;    // 文字数カウンタ
        $chunklen   = 30;   // 元文字列長
        $encoded    = "";   // 変換文字列

        //[ 各種折りたたみについて繰り返す。 ]
        while(strlen($str) > 0 && $str != JIS_ESC_START && $cnt < 100)
        {
            $is_ascii = true; $cnt++;
            $second_byte = false;
            $len = min($chunklen, strlen($str)) - strlen(JIS_ESC_END) + 1;

            //[ 文字列を前から(JIS_END-1)文字を残して探索 ]
            for ($i = 0;$i<$len;$i++)
            {
                //[ JISのエスケープコードならasciiと非asciiを切り替える。 ]
                if ($str{$i} == "\x1B")
                {
                    $is_ascii = !$is_ascii;
                    $second_byte = false;
                }
                //[ マルチバイトならば1文字目と2文字目を毎回きりかえ ]
                else if (!$is_ascii)
                {
                    $second_byte = !$second_byte;
                }
            }

            //[ 折りたたみがasciiで終わっている場合、最大長で切り取る ]
            if ($is_ascii)
            {
                //[ 最後から2文字目がasciiなら行に追記 ]
                if ($str{$i} != "\x1B")
                {
                    $len++;
                    //[ 最後から1文字目がasciiなら行に追記 ]
                    if ($str{$i+1} != "\x1B")
                    {
                        $len++;
                    }
                }

                $chunked = substr($str, 0, $len);
                $str = substr($str, $len);
            }
            //[ 最後の3文字がascii->非asciiのエスケープと重なっている場合、その前まで切り取る。 ]
            else if (strpos(substr($str, $len-3),JIS_ESC_START) !== false)
            {
                $len = strrpos($str, JIS_ESC_START);
                $chunked = substr($str, 0, $len);
                $str = substr($str, $len);
            }
            //[ 折りたたみが非asciiの1文字目で終わっている場合、最大長-1で切り取り、ESCを追加する ]
            else if ($second_byte)
            {
                $len = min($chunklen-strlen(JIS_ESC_END), strlen($str));
                $chunked = substr($str, 0, $len);
                if (substr($chunked, -strlen(JIS_ESC_END)) != JIS_ESC_END)
                {
                    $chunked .= JIS_ESC_END;
                }
                $str = JIS_ESC_START.substr($str, $len);
            }
            //[ 折りたたみが非asciiの2文字目で終わっている場合、最大長-2で切り取り、ESCを追加する ]
            else
            {
                $len = min($chunklen-strlen(JIS_ESC_END)-1, strlen($str));
                $chunked = substr($str, 0, $len);
                if (substr($chunked, -strlen(JIS_ESC_END)) != JIS_ESC_END)
                {
                    $chunked .= JIS_ESC_END;
                }
                $str = JIS_ESC_START.substr($str, $len);
            }

            // ２行目以降の場合、改行文字列を追加する
            //（最終行の行末には改行をつけない→ヘッダー作成時に改行が付く為）
            if( !empty($encoded) ){
                $encoded .= $ending;
            }
            $encoded .= " =?ISO-2022-JP?B?".base64_encode($chunked)."?=";
        }

        return $encoded;
    }

    //--------------------------------------------------------------------------
    //--    関数名： rm_insertCR                                              --
    //--    Method： rm_insertCR                                              --
    //--------------------------------------------------------------------------
    //--  MODIFIED        YYYY-MM-DD REV    - NOTES                           --
    //--  NEOJAPAN        2007-05-30 1.00   - CREATE NEW                      --
    //--------------------------------------------------------------------------
    /**
     * 1行の文字が内部規定の最大文字数を超えたら改行コードを挿入
     * RFCの規定では1行、998バイトまで
     *
     * @access private
     * @param  string  $body        [IN]:本文
     * @return string  変換後の本文
     */
    function rm_insertCR($body)
    {
        // 本文を行単位の配列に変換
// 2007.6.8 改行が増殖する
//        $lines = mb_split ("\n", $body);
//
        $body2 = mb_ereg_replace( "\r\n", "\n", $body );
        $lines = mb_split ("\n", $body2);
// 2007.6.8 改行が増殖する

        $new_body = "";

        // 行数分ループ
        foreach ($lines as $one_line) {
            $new_line = "";

            // 1行の文字数を取得
            $line_length = mb_strlen($one_line);
            // 1行が最大サイズを超える場合
            if ($line_length > NEOMAILER_MAX_LINE_LENGTH) {
                // 分割行数を求める
                if (($line_length % NEOMAILER_MAX_LINE_LENGTH) == 0) {
                     $loop_cnt = $line_length / NEOMAILER_MAX_LINE_LENGTH;
                } else {
                     $loop_cnt = ceil($line_length / NEOMAILER_MAX_LINE_LENGTH);
                }

                $offset = 0;
                // 分割対象の文字の最後に改行コードを挿入する
                for ($nCnt = 1; $nCnt <= $loop_cnt; $nCnt++) {
                     $new_line .= mb_substr ($one_line, $offset, NEOMAILER_MAX_LINE_LENGTH) . "\r\n";
                     $offset = NEOMAILER_MAX_LINE_LENGTH * $nCnt;
                }
            }
            else {
                $new_line = $one_line . "\r\n";
            }

            $new_body .= $new_line;
        }
        return $new_body;
    }
}
?>